//---------------------------------------------------------------------------
// Copyright 2006-2020 Joe Lowe
//
// Permission is granted to any person obtaining a copy of this Software,
// to deal in the Software without restriction, including the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and sell copies of
// the Software.
//
// The above copyright and permission notice must be left intact in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS WITHOUT WARRANTY.
//---------------------------------------------------------------------------
// file name:  ptstdint.h
// created:    2006.11.12
//---------------------------------------------------------------------------
#ifndef PTSTDINT_H
#define PTSTDINT_H
#ifdef __cplusplus
extern "C" {
#endif

   // Subset of definitions from ISO C99 stdint.h, for systems/compilers
   // that do not supply the header.

#if !defined(SIZE_MAX)
#if defined(_MSCVER) && defined(_WIN64)
   // MSVC is only modern system/compiler where size of long is not
   // 64 bit on 64 bit systems.
#define SIZE_MAX _UI64_MAX
#elif LONG_MAX == INT_MAX
#define SIZE_MAX UINT_MAX
#else
#define SIZE_MAX ULONG_MAX
#endif
#endif

#ifndef __BIT_TYPES_DEFINED__ // linux/kernel.h
typedef        signed char  int8_t   ;
typedef      unsigned char  uint8_t  ;
typedef              short  int16_t  ;
typedef     unsigned short  uint16_t ;
typedef                int  int32_t  ;
typedef           unsigned  uint32_t ;
#if LONG_MAX > INT_MAX
typedef               long  int64_t  ;
typedef      unsigned long  uint64_t ;
#elif defined(_MSC_VER)
typedef            __int64  int64_t  ;
typedef   unsigned __int64  uint64_t ;
#else
typedef          long long  int64_t  ;
typedef unsigned long long  uint64_t ;
#endif
#elif LINUX_VER >= 20624
#define _UINTPTR_T_DEFINED
#endif // __BIT_TYPES_DEFINED__
#if !defined(_INTPTR_T_DEFINED) && !defined(__intptr_t_defined)
#define _INTPTR_T_DEFINED
#define __intptr_t_defined
#if SIZE_MAX == UINT_MAX
typedef                int  intptr_t ;
#if !defined(_UINTPTR_T_DEFINED)
typedef           unsigned  uintptr_t;
#endif
#elif LONG_MAX > INT_MAX
typedef               long  intptr_t ;
#if !defined(_UINTPTR_T_DEFINED)
typedef      unsigned long  uintptr_t;
#endif
#elif defined(_MSC_VER)
typedef            __int64  intptr_t ;
#if !defined(_UINTPTR_T_DEFINED)
typedef   unsigned __int64  uintptr_t;
#endif
#else
typedef          long long  intptr_t ;
#if !defined(_UINTPTR_T_DEFINED)
typedef unsigned long long  uintptr_t;
#endif
#endif
#endif

#if defined(_MSC_VER) && !defined(INT8_MIN)
   // Microsoft intsafe.h system header is ignorant of C99 stdint.h.
   // These definitions have to match to avoid compiler errors.
#define INT8_MIN        (-127i8 - 1)
#define INT8_MAX        127i8
#define UINT8_MAX       0xffui8
#define INT16_MIN       (-32767i16 - 1)
#define INT16_MAX       32767i16
#define UINT16_MAX      0xffffui16
#define INT32_MIN       (-2147483647i32 - 1)
#define INT32_MAX       2147483647i32
#define UINT_MAX        0xffffffff
#define UINT32_MAX      0xffffffffui32
#define INT64_MIN       (-9223372036854775807i64 - 1)
#define INT64_MAX       9223372036854775807i64
#define UINT64_MAX      0xffffffffffffffffui64
#endif

#ifndef INT8_MIN
#define    INT8_MIN      SCHAR_MIN
#define    INT8_MAX      SCHAR_MAX
#define   UINT8_MAX      UCHAR_MAX
#define   INT16_MIN       SHRT_MIN
#define   INT16_MAX       SHRT_MAX
#define  UINT16_MAX      USHRT_MAX
#define   INT32_MIN        INT_MIN
#define   INT32_MAX        INT_MAX
#define  UINT32_MAX       UINT_MAX
#if LONG_MAX > INT_MAX
#define   INT64_MIN       LONG_MIN
#define   INT64_MAX       LONG_MAX
#define  UINT64_MAX      ULONG_MAX
#else
#define   INT64_MIN      (-9223372036854775807LL-1)
#define   INT64_MAX        9223372036854775807LL
#define  UINT64_MAX        0xFFFFFFFFFFFFFFFFULL
#endif
#endif
#if !defined(INTPTR_MIN)
#if SIZE_MAX == UINT_MAX
#define   INTPTR_MIN       INT_MIN
#define   INTPTR_MAX       INT_MAX
#ifndef UINTPTR_MAX
#define  UINTPTR_MAX      UINT_MAX
#endif
#elif LONG_MAX > INT_MAX
#define   INTPTR_MIN      LONG_MIN
#define   INTPTR_MAX      LONG_MAX
#ifndef UINTPTR_MAX
#define  UINTPTR_MAX     ULONG_MAX
#endif
#elif defined(_MSC_VER)
#define   INTPTR_MIN     INT64_MIN
#define   INTPTR_MAX     INT64_MAX
#define  UINTPTR_MAX    UINT64_MAX
#else
#define   INTPTR_MIN     (-9223372036854775807LL-1)
#define   INTPTR_MAX       9223372036854775807LL
#define  UINTPTR_MAX       0xFFFFFFFFFFFFFFFFULL
#endif
#endif
#if !defined(PTRDIFF_MIN)
#define PTRDIFF_MIN     INTPTR_MIN
#define PTRDIFF_MAX     INTPTR_MAX
#endif

#if !defined(INT8_C)
#define INT8_C(c)   c
#define UINT8_C(c)  c##U
#define INT16_C(c)  c
#define UINT16_C(c) c##U
#define INT32_C(c)  c
#define UINT32_C(c) c##U
#if LONG_MAX > INT_MAX
#define INT64_C(c)  c##L
#define UINT64_C(c) c##UL
#elif defined(_MSC_VER)
#define INT64_C(c)  c##I64
#define UINT64_C(c) c##UI64
#else
#define INT64_C(c)  c##LL
#define UINT64_C(c) c##ULL
#endif
#endif

#ifdef __cplusplus
} // extern "C"
#endif
#endif
