//----------------------------------------------------------------------------
// Copyright 2001-2023 Joe Lowe
//
// Permission is granted to any person obtaining a copy of this Software,
// to deal in the Software without restriction, including the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and sell copies of
// the Software.
//
// The above copyright and permission notice must be left intact in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS WITHOUT WARRANTY.
//----------------------------------------------------------------------------
// file name:  trimp.c
// created:    2001.06.05
//----------------------------------------------------------------------------
#include "ptlibc.h"
#include "portableint.h"
#include "traceio.h"

_TRPRIV TRCLIENT traceClient = 0;
_TRPRIV TRACEEXPORTS* traceExports = nullptr;
_TRPRIV int/*bool*/ traceNoDetach = false;
static TRCHANNEL traceDefaultChannel = TRDEF;

static TRCHANNEL trfixchannel(TRCHANNEL channel)
{
   return (channel == TRDEF)?traceDefaultChannel:channel;
}

_TRPRIV TRCHANNEL _TRCDECL tropen(const char* name)
{
   TRCHANNEL channel = 0;

   trautoattach();
   if(traceExports && traceExports->tropen)
   {
      channel = traceExports->tropen(traceClient,name);
   }

   return channel;
}

_TRPRIV TRCHANNEL _TRCDECL trwopen(const wchar_t* name)
{
   TRCHANNEL channel = 0;

   trautoattach();
   if(traceExports && traceExports->trwopen)
   {
      channel = traceExports->trwopen(traceClient,name);
   }

   return channel;
}

_TRPRIV TRCHANNEL _TRCDECL trdup(TRCHANNEL oldChannel)
{
   TRCHANNEL newChannel = 0;

   trautoattach();
   if(traceExports && traceExports->trdup)
   {
      newChannel = traceExports->trdup(traceClient,oldChannel);
   }

   return newChannel;
}

_TRPRIV void _TRCDECL trclose(TRCHANNEL channel)
{
   if(traceExports && traceExports->trclose)
   {
      traceExports->trclose(traceClient,channel);
   }
}

_TRPRIV void _TRCDECL trdefault(TRCHANNEL channel)
{
   // if(traceExports && traceExports->trdefault)
   // {
   //    traceExports->trdefault(traceClient,channel);
   // }
   traceDefaultChannel = channel;
}

_TRPRIV void _TRCDECL trreset(TRCHANNEL channel)
{
   if(traceExports && traceExports->trreset)
   {
      traceExports->trreset(traceClient,trfixchannel(channel));
   }
}

_TRPRIV void _TRCDECL trlist(char* buffer,size_t maxChars)
{
   trautoattach();
   if(traceExports && traceExports->trlist)
   {
      traceExports->trlist(traceClient,buffer,maxChars);
   }
}

_TRPRIV void _TRCDECL trwlist(wchar_t* buffer,size_t maxChars)
{
   trautoattach();
   if(traceExports && traceExports->trwlist)
   {
      traceExports->trwlist(traceClient,buffer,maxChars);
   }
}

_TRPRIV void _TRCDECL trinfo(TRCHANNEL channel,size_t* startLine,
   size_t* endLine,size_t* endLineLen)
{
   if(traceExports && traceExports->trinfo)
   {
      traceExports->trinfo(traceClient,trfixchannel(channel),startLine,
         endLine,endLineLen);
   }
}

_TRPRIV void _TRCDECL trread(TRCHANNEL channel,size_t line,char* buffer,
   size_t maxChars)
{
   if(traceExports && traceExports->trread)
   {
      traceExports->trread(traceClient,trfixchannel(channel),line,buffer,
         maxChars);
   }
}

_TRPRIV void _TRCDECL trwread(TRCHANNEL channel,size_t line,wchar_t* buffer,
   size_t maxChars)
{
   if(traceExports && traceExports->trwread)
   {
      traceExports->trwread(traceClient,trfixchannel(channel),line,buffer,
         maxChars);
   }
}

_TRPRIV void _TRCDECL trprint(TRCHANNEL channel,const char* value)
{
   trautoattach();
   if(channel && traceExports && traceExports->trprint)
   {
      traceExports->trprint(traceClient,trfixchannel(channel),value);
   }
}

_TRPRIV void _TRCDECL trwprint(TRCHANNEL channel,const wchar_t* value)
{
   trautoattach();
   if(channel && traceExports && traceExports->trwprint)
   {
      traceExports->trwprint(traceClient,trfixchannel(channel),value);
   }
}

_TRPRIV void _TRCDECL trvprintf(TRCHANNEL channel,const char* format,
   va_list args)
{
   trautoattach();
   if(channel && traceExports && traceExports->trvprintf)
   {
      traceExports->trvprintf(traceClient,trfixchannel(channel),format,args);
   }
}

_TRPRIV void _TRCDECL trvwprintf(TRCHANNEL channel,const wchar_t* format,
   va_list args)
{
   trautoattach();
   if(channel && traceExports && traceExports->trvwprintf)
   {
      traceExports->trvwprintf(traceClient,trfixchannel(channel),format,
         args);
   }
}

_TRPRIV void _TRCDECL trprintf(TRCHANNEL channel,const char* format,...)
{
   trautoattach();
   if(channel && traceExports && traceExports->trvprintf)
   {
      va_list args;
      va_start(args,format);
      traceExports->trvprintf(traceClient,trfixchannel(channel),format,args);
      va_end(args);
   }
}

_TRPRIV void _TRCDECL trwprintf(TRCHANNEL channel,const wchar_t* format,...)
{
   trautoattach();
   if(channel && traceExports && traceExports->trvwprintf)
   {
      va_list args;
      va_start(args,format);
      traceExports->trvwprintf(traceClient,trfixchannel(channel),format,
         args);
      va_end(args);
   }
}

_TRPRIV void _TRCDECL trcrash(void)
{
   trautoattach();
   if(traceExports && traceExports->trcrash)
   {
      traceExports->trcrash(traceClient);
   }
}
